#ifndef _SPE_H

#include_next <spe.h>

libc_hidden_proto (strtosfix16)
libc_hidden_proto (strtosfix32)
libc_hidden_proto (strtosfix64)
libc_hidden_proto (strtoufix16)
libc_hidden_proto (strtoufix32)
libc_hidden_proto (strtoufix64)

#endif
